use std::borrow::Borrow;
use std::collections::{HashMap, HashSet};
use std::fmt::{self, Debug};
use std::hash::Hash;

pub struct MapOfSet<K, V>(HashMap<K, HashSet<V>>);

impl<K: Debug, V: Debug> Debug for MapOfSet<K, V> {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        self.0.fmt(f)
    }
}

impl<K, V> Default for MapOfSet<K, V> {
    fn default() -> Self {
        Self(HashMap::new())
    }
}

impl<K: Hash + Eq, V: Hash + Eq> MapOfSet<K, V> {
    pub fn insert_item(&mut self, key: K, value: V) {
        if let Some(container) = self.0.get_mut(&key) {
            container.insert(value);
        } else {
            let mut container = HashSet::new();
            container.insert(value);
            self.0.insert(key, container);
        }
    }

    pub fn get<Q: ?Sized>(&self, key: &Q) -> Option<&HashSet<V>>
    where
        K: Borrow<Q>,
        Q: Hash + Eq,
    {
        self.0.get(key)
    }

    pub fn key_contains<QK: ?Sized, QV: ?Sized>(
        &self,
        key: &QK,
        value: &QV,
    ) -> bool
    where
        K: Borrow<QK>,
        QK: Hash + Eq,
        V: Borrow<QV>,
        QV: Hash + Eq,
    {
        match self.0.get(key) {
            Some(container) => container.contains(value),
            None => false,
        }
    }

    pub fn iter(&self) -> impl Iterator<Item = (&K, &HashSet<V>)> {
        self.0.iter()
    }
}
